@@ -9,6 +9,7 @@ import torchaudio
 from torchaudio.functional import resample
 
 import torch
+import torch.nn.functional as F
 from torch.nn.utils.rnn import pad_sequence
 from torch.utils.data import Dataset, DataLoader
 
@@ -24,6 +25,10 @@ def exists(val):
 def cast_tuple(val, length = 1):
     return val if isinstance(val, tuple) else ((val,) * length)
 
+# type
+
+OptionalIntOrTupleInt = Optional[Union[int, Tuple[Optional[int], ...]]]
+
 # dataset functions
 
 @beartype
@@ -32,9 +37,9 @@ class SoundDataset(Dataset):
         self,
         folder,
         exts = ['flac', 'wav'],
-        max_length = None,
-        target_sample_hz: Optional[Union[int, Tuple[Optional[int], ...]]] = None,
-        seq_len_multiple_of = None
+        max_length: OptionalIntOrTupleInt = None,
+        target_sample_hz: OptionalIntOrTupleInt = None,
+        seq_len_multiple_of: OptionalIntOrTupleInt = None
     ):
         super().__init__()
         path = Path(folder)
@@ -44,10 +49,14 @@ class SoundDataset(Dataset):
         assert len(files) > 0, 'no sound files found'
 
         self.files = files
-        self.max_length = max_length
 
         self.target_sample_hz = cast_tuple(target_sample_hz)
-        self.seq_len_multiple_of = seq_len_multiple_of
+        num_outputs = len(self.target_sample_hz)
+
+        self.max_length = cast_tuple(max_length, num_outputs)
+        self.seq_len_multiple_of = cast_tuple(seq_len_multiple_of, num_outputs)
+
+        assert len(self.max_length) == len(self.target_sample_hz) == len(self.seq_len_multiple_of)
 
     def __len__(self):
         return len(self.files)
@@ -57,35 +66,50 @@ class SoundDataset(Dataset):
 
         data, sample_hz = torchaudio.load(file)
         
-        if data.size(1) > self.max_length:
-            max_start = data.size(1) - self.max_length
-            start = torch.randint(0, max_start, (1, ))
-            data = data[:, start:start + self.max_length]
-
-        else:
-            data = torch.nn.functional.pad(data, (0, self.max_length - data.size(1)), 'constant')
-        
-        data = rearrange(data, '1 ... -> ...')
-
         num_outputs = len(self.target_sample_hz)
         data = cast_tuple(data, num_outputs)
 
         # resample if target_sample_hz is not None in the tuple
 
-        data = tuple((resample(d, sample_hz, target_sample_hz) if exists(target_sample_hz) else d) for d, target_sample_hz in zip(data, self.target_sample_hz))
+        data_tuple = tuple((resample(d, sample_hz, target_sample_hz) if exists(target_sample_hz) else d) for d, target_sample_hz in zip(data, self.target_sample_hz))
 
-        if exists(self.max_length):
-            data = tuple(d[:self.max_length] for d in data)
+        output = []
 
-        if exists(self.seq_len_multiple_of):
-            data = tuple(curtail_to_multiple(d, self.seq_len_multiple_of) for d in data)
+        # process each of the data resample at different frequencies individually
 
-        data = tuple(d.float() for d in data)
+        for data, max_length, seq_len_multiple_of in zip(data_tuple, self.max_length, self.seq_len_multiple_of):
+            audio_length = data.size(1)
+
+            # pad or curtail
+
+            if audio_length > max_length:
+                max_start = audio_length - max_length
+                start = torch.randint(0, max_start, (1, ))
+                data = data[:, start:start + max_length]
+
+            else:
+                data = F.pad(data, (0, max_length - audio_length), 'constant')
+
+            data = rearrange(data, '1 ... -> ...')
+
+            if exists(max_length):
+                data = data[:max_length]
+
+            if exists(seq_len_multiple_of):
+                data = curtail_to_multiple(data, seq_len_multiple_of)
+
+            output.append(data.float())
+
+        # cast from list to tuple
+
+        output = tuple(output)
+
+        # return only one audio, if only one target resample freq
 
         if num_outputs == 1:
-            return data[0]
+            return output[0]
 
-        return data
+        return output
 
 # dataloader functions
 
